#![cfg(target_arch = "x86_64")]
#![allow(dead_code)]

use x86_64::structures::gdt::{Descriptor, GlobalDescriptorTable};
use x86_64::registers::segmentation::{CS, SS, Segment, SegmentSelector};
use x86_64::PrivilegeLevel;
use spin::Mutex;

static GDT: Mutex<GlobalDescriptorTable> = Mutex::new(GlobalDescriptorTable::new());

fn add_entry(descriptor: Descriptor)
{
    GDT.lock().add_entry(descriptor);
}

pub fn init()
{
    add_entry(Descriptor::kernel_code_segment());
    add_entry(Descriptor::kernel_data_segment());
    add_entry(Descriptor::user_code_segment());
    add_entry(Descriptor::user_data_segment());
    // TODO: add a TSS.
}

pub fn load_and_reload_segment_registers()
{
    unsafe {
        GDT.lock().load_unsafe();
        CS::set_reg(SegmentSelector::new(1, PrivilegeLevel::Ring0));
        SS::set_reg(SegmentSelector::new(2, PrivilegeLevel::Ring0));
    }
}