use x86_64::structures::idt::{InterruptDescriptorTable, InterruptStackFrame, PageFaultErrorCode};
use crate::try_println;
use core::arch::asm;

static mut IDT: InterruptDescriptorTable = InterruptDescriptorTable::new();

extern "x86-interrupt" fn breakpoint_handler(stack_frame: InterruptStackFrame)
{
    try_println!("Breakpoint!\n{:#?}", stack_frame);
}

extern "x86-interrupt" fn page_fault_handler(stack_frame: InterruptStackFrame, faulting_address: PageFaultErrorCode)
{
    try_println!("Page fault at {:#?}\n{:#?}", faulting_address, stack_frame);
}

extern "x86-interrupt" fn double_fault_handler(stack_frame: InterruptStackFrame, _error_code: u64) -> ! {
    panic!("Double fault!\n{:#?}", stack_frame);
}

pub fn load()
{
    unsafe { 
        IDT.breakpoint.set_handler_fn(breakpoint_handler);
        IDT.page_fault.set_handler_fn(page_fault_handler);
        IDT.double_fault.set_handler_fn(double_fault_handler);
        IDT.load(); 
    }
}

pub fn enable()
{
    unsafe { asm!("sti"); }
}

pub fn disable()
{
    unsafe { asm!("cli"); }
}