#![cfg(target_arch = "x86_64")]
#![allow(dead_code)]

pub const COM1: u16 = 0x3f8;

use core::arch::asm;

pub unsafe fn outportb(port: u16, value: u8) -> () {
    asm!("out dx, al", in("al") value, in("dx") port);
}

pub unsafe fn inportb(port: u16) -> u8 {
    let value: u8;
    asm!("in al, dx", out("al") value, in("dx") port);
    return value;
}

pub unsafe fn io_delay() -> () {
    outportb(0x80,0);
}

pub mod serial {
    use super::*;

    unsafe fn wait() -> () {
        while (inportb(COM1 + 5) & 0x20) == 0 {
            asm!("pause");
        }
    }
    
    pub fn putchar(c: u8) -> () {
        unsafe { wait(); }
        unsafe { outportb(COM1, c); }
    }
}